@@ -22,6 +22,8 @@ from time import time
 
 Graph = Database(MapMultidigraph)
 
+_autostate = '-1'
+
 class EnumByName(Enum):
     def _generate_next_value_(name, start, count, last_values):
         return name
@@ -33,11 +35,12 @@ class Speaker(EnumByName):
 class DialogueFlow:
 
     Speaker = Speaker
-    _autostate = '-1'
+
     @classmethod
     def autostate(cls):
-        cls._autostate = str(int(cls._autostate) + 1)
-        return cls._autostate
+        global _autostate
+        _autostate = str(int(_autostate) + 1)
+        return _autostate
 
     def __init__(self, initial_state: Union[Enum, str, tuple], initial_speaker = Speaker.SYSTEM,
                  macros: Dict[str, Macro] =None, kb: Union[KnowledgeBase, str, List[str]] =None):
@@ -91,7 +94,8 @@ class DialogueFlow:
             'CLR': Clear(),
             'NER': NamedEntity(),
             'POS': PartOfSpeech(),
-            'LEM': Lemma()
+            'LEM': Lemma(),
+            'SCORE': Score()
         }
         if macros:
             self._macros.update(macros)
@@ -240,11 +244,22 @@ class DialogueFlow:
                         self.add_state(target)
                         self.set_error_successor(target, root)
                 if speaker == Speaker.USER:
-                    self.add_user_transition(source, target, natex, score=score)
+                    if self.has_transition(source, target, Speaker.USER):
+                        intermediate = self.autostate()
+                        self.add_state(intermediate, target, user_multi_hop=True)
+                        self.add_user_transition(source, intermediate, natex, score=score)
+                    else:
+                        self.add_user_transition(source, target, natex, score=score)
                 elif speaker == Speaker.SYSTEM:
-                    self.add_system_transition(source, target, natex, score=score)
+                    if self.has_transition(source, target, Speaker.SYSTEM):
+                        intermediate = self.autostate()
+                        self.add_state(intermediate, system_multi_hop=True)
+                        self.add_system_transition(intermediate, target, '')
+                        self.add_system_transition(source, intermediate, natex, score=score)
+                    else:
+                        self.add_system_transition(source, target, natex, score=score)
 
-        # swtich turn (will be switched back if multi hop detected on next recursive call)
+        # switch turn (will be switched back if multi hop detected on next recursive call)
         if speaker == Speaker.USER:
             speaker = Speaker.SYSTEM
         elif speaker == Speaker.SYSTEM:
@@ -288,7 +303,12 @@ class DialogueFlow:
                 print()
                 generation = None
             if generation is not None:
-                transition_options[(generation, transition, vars)] = settings.score
+                if '__score__' in vars:
+                    score = vars['__score__']
+                    del vars['__score__']
+                else:
+                    score = settings.score
+                transition_options[(generation, transition, vars)] = score
             t2 = time()
             if debugging:
                 print('Transition {} evaluated in {:.5f}'.format(transition, t2-t1))
@@ -366,7 +386,12 @@ class DialogueFlow:
             if match:
                 if debugging:
                     print('Transition {} matched "{}"'.format(transition[:2], natural_language))
-                transition_options.append((settings.score, transition, vars))
+                if '__score__' in vars:
+                    score = vars['__score__']
+                    del vars['__score__']
+                else:
+                    score = settings.score
+                transition_options.append((score, transition, vars))
             t2 = time()
             if debugging:
                 print('Transition {} evaluated in {:.5f}'.format(transition, t2-t1))
@@ -731,3 +756,6 @@ class DialogueFlow:
             response, score = result
             self._response = response, vars, self._state, score
 
+    def knowledge_base(self):
+        return self._kb
+
